import logging
from functools import wraps

import click
import jwt

from .cli_tools import CliTools

logger = logging.getLogger(__name__)


def config_params(fn):
    @wraps(fn)
    @click.option('token', '--token', '--ml-token', help='Resource identifying token generated by MissingLink')
    @click.option('ws_server', '--ml-server', '--ws', help='Resource identifying token generated by MissingLink')
    @click.option('--ml-config-prefix', default=None, help='')
    @click.option('--ml-config-file', help='')
    @click.option('backend_base_url', '--ml-backend', '--backend', default='https://missinglinkai.appspot.com/_ah/api/missinglink/v1', help='backend base url')
    @click.option('--ssh-private-key', help='Path to the private key to be used when working with source control.')
    @click.option('--env', type=(str, str), multiple=True, help='Additional environment variables to be set on containers started by the resource manager')
    @click.option('--mount', type=(str, str), multiple=True, help='Additional volume mappings to be performed on containers started by the resource manager.')
    @click.option('--docker-auth', type=(str, str, str), multiple=True, help='Docker HOST USER PASS (in this order!) to be used before pulling new images. ECR are still authenticated automatically')
    def decorated(**kwargs):
        new_conf = ConfigBuilder.build_config(kwargs)
        kwargs.update(new_conf)
        return fn(**new_conf)

    return decorated


class ConfigBuilder(object):
    @classmethod
    def create(cls, active_config, **kwargs):
        return cls(active_config, **kwargs)

    def __init__(self, active_config, **kwargs):
        self.active_config = active_config
        self.kwargs = kwargs

    @classmethod
    def build_config(cls, kwargs):
        new_config = {
            'token': kwargs.pop('token', None),
            'ws_server': kwargs.pop('ws_server', None),
            'ssh_private_key': kwargs.pop('ssh_private_key', None),
            'ml_config_file': kwargs.pop('ml_config_file', None),
            'ml_config_prefix': kwargs.pop('ml_config_prefix', None),
            'backend_base_url': kwargs.pop('backend_base_url', None),
            'env': {t[0]: t[1] for t in kwargs.pop('env', [])},
            'mount': {t[0]: t[1] for t in kwargs.pop('mount', [])},
            'docker_auth': {t[0]: {'user': t[1], 'pass': t[2]} for t in kwargs.pop('docker_auth', [])}}
        return {k: v for k, v in new_config.items() if v}

    def _conf_ws_server(self):
        ws_server = self.kwargs.pop('ws_server', None)
        if ws_server is not None:
            self.active_config.general.ws_server = ws_server
            logger.info('Missing Link Server is set to %s', ws_server)
            click.echo('Missing Link Server is set to {}'.format(ws_server))

    def _conf_backend_base_url(self):
        backend_base_url = self.kwargs.pop('backend_base_url', None)
        if backend_base_url:
            self.active_config.general.backend_base_url = backend_base_url
            logger.info('Missing Link backend_base_url is set to %s', backend_base_url)
            click.echo('Missing Link backend_base_url is set to {}'.format(backend_base_url))

    def _conf_ml_token(self):
        ml_token = self.kwargs.pop('token', None)

        if ml_token is not None:
            tok = jwt.decode(ml_token, verify=False)
            sid = tok.get("uid")
            self.active_config.general.cluster_id = sid
            self.active_config.general.jwt = ml_token
            self.active_config.general.save()
            logger.info('Missing Link Token is set. Resource Id: %s', sid)
            click.echo('Missing Link Token is set. Resource Id: {}'.format(sid))

    def _conf_ssh_key(self):
        ssh_private_key = self.kwargs.pop('ssh_private_key', None)
        if ssh_private_key is not None:
            CliTools.populate_ssh_key(self.active_config, ssh_private_key)
            logger.info('SSH key is set')

    def _conf_ml_creds(self):
        ml_config_prefix = self.kwargs.pop('ml_config_prefix', None)
        ml_config_file = self.kwargs.pop('ml_config_file', None)
        if ml_config_file is not None:
            if ml_config_prefix == str(None):
                ml_config_prefix = None

            self.active_config.general.ml_prefix = ml_config_prefix
            CliTools.save_ml_config(self.active_config, config_prefix=ml_config_prefix, config_data=ml_config_file)
            logger.info('Credentials set')

    def _conf_env(self):
        env = self.kwargs.pop('env', {})
        if env:
            self.active_config.general.env = env
            logger.info('env is set: %s items', len(env))

    def _conf_mounts(self):
        mounts = self.kwargs.pop('mount', {})
        if mounts:
            self.active_config.general.mount = mounts
            logger.info('mounts is set: %s items', len(mounts))

    def _conf_docker_auths(self):
        docker_auths = self.kwargs.pop('docker_auth', {})
        if docker_auths:
            self.active_config.general.docker_auth = docker_auths
            logger.info('docker_auths is set: %s items', len(docker_auths))

    def parse_and_save(self):
        self._conf_ws_server()
        self._conf_backend_base_url()
        self._conf_ml_token()
        self._conf_ssh_key()
        self._conf_ml_creds()
        self._conf_env()
        self._conf_mounts()
        self._conf_docker_auths()
        self.active_config.general.save()
        click.echo(f"{self.active_config.config_path} Saved")
